#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.models.rattail`` -- Models for 'rattail' Extension
"""

from __future__ import absolute_import

from django.db import models

from rattail.gpc import GPC
from rattail.django.rattail.models import Model, uuid_field
from rattail.django.rattail.models.contact import (
    ContactInfoManager, PhoneNumber, EmailAddress)


__all__ = ['Store', 'StorePhoneNumber', 'StoreEmailAddress',
           'Department', 'Subdepartment', 'Category',
           'Brand', 'Product']


class GPCField(models.BigIntegerField):
    """
    Custom type for the ``Product.upc`` field.
    """

    description = "Global Product Classification (GPC)"

    __metaclass__ = models.SubfieldBase

    def to_python(self, value):
        if value is None:
            return None
        return GPC(value)

    def get_prep_value(self, value):
        if value is None:
            return None
        return int(value)


class Store(Model):
    """
    Represents a store (physical or otherwise) within the organization.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('stores')

    uuid = uuid_field()
    id = models.CharField(max_length=10, blank=True, null=True, verbose_name="ID")
    name = models.CharField(max_length=100, blank=True, null=True)

    def __repr__(self):
        return "<Store: %s, %s>" % (self.id, self.name)

    def __unicode__(self):
        return unicode(self.name or '')


class StoreContactInfo(models.Model):
    """
    Base class for store contact models (phone number, etc.).
    """

    class Meta:
        abstract = True

    store = models.ForeignKey(Store, db_column='parent_uuid')
    objects = ContactInfoManager('Store')

    def save(self, *args, **kwargs):
        self.parent_type = 'Store'
        super(StoreContactInfo, self).save(*args, **kwargs)


class StorePhoneNumber(PhoneNumber, StoreContactInfo):
    """
    Represents a phone (or fax) number associated with a :class:`Store`.
    """


class StoreEmailAddress(EmailAddress, StoreContactInfo):
    """
    Represents an email address associated with a :class:`Store`.
    """


class Department(Model):
    """
    Represents an organizational department.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('departments')

    uuid = uuid_field()
    number = models.IntegerField(blank=True, null=True)
    name = models.CharField(max_length=30, blank=True, null=True)

    def __repr__(self):
        return "<Department: %s>" % self.name

    def __unicode__(self):
        return unicode(self.name or '')


class Subdepartment(Model):
    """
    Represents an organizational subdepartment.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('subdepartments')

    uuid = uuid_field()
    number = models.IntegerField(blank=True, null=True)
    name = models.CharField(max_length=30, blank=True, null=True)
    department = models.ForeignKey(Department, db_column='department_uuid', blank=True, null=True)

    def __repr__(self):
        return "<Subdepartment: %s>" % self.name

    def __unicode__(self):
        return unicode(self.name or '')


class Category(Model):
    """
    Represents an organizational category for products.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('categories')
        verbose_name_plural = "Categories"

    uuid = uuid_field()
    number = models.IntegerField(blank=True, null=True)
    name = models.CharField(max_length=50, blank=True, null=True)
    department = models.ForeignKey(Department, db_column='department_uuid', blank=True, null=True)

    def __repr__(self):
        return "<Category: %s, %s>" % (self.number, self.name)

    def __unicode__(self):
        return unicode(self.name or '')


class Brand(Model):
    """
    Represents a brand or similar product line.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('brands')

    uuid = uuid_field()
    name = models.CharField(max_length=100, blank=True, null=True)

    def __repr__(self):
        return "<Brand: %s>" % self.name

    def __unicode__(self):
        return unicode(self.name or '')


class Product(Model):
    """
    Represents a product for sale and/or purchase.
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('products')

    uuid = uuid_field()
    upc = GPCField(db_index=True, verbose_name="UPC")
    department = models.ForeignKey(Department, db_column='department_uuid', blank=True, null=True)
    subdepartment = models.ForeignKey(Subdepartment, db_column='subdepartment_uuid', blank=True, null=True)
    category = models.ForeignKey(Category, db_column='category_uuid', blank=True, null=True)
    brand = models.ForeignKey(Brand, db_column='brand_uuid', blank=True, null=True)
    description = models.CharField(max_length=60, blank=True, null=True)
    description2 = models.CharField(max_length=60, blank=True, null=True, verbose_name="Description Line 2")
    size = models.CharField(max_length=30, blank=True, null=True)
    unit_of_measure = models.CharField(max_length=4, blank=True, null=True, verbose_name="Unit of Measure")

    def __repr__(self):
        return "<Product: %s>" % self.description

    def __unicode__(self):
        return unicode(self.description or '')
